"""
Flask-MarcoPolo

$project_name.views.error

Error views for your application

In your other views, error pages can be invoked by using abort(404, message)
"""

from flask.ext.marcopolo import MarcoPolo


class Error(MarcoPolo):
    """
    Error Views
    """
    @classmethod
    def register(cls, app, **kwargs):
        super(cls, cls).register(app, **kwargs)

        # Bind the error to app
        @app.errorhandler(400)
        def error_400(error):
            return cls.error_400(error)

        # Bind the error to app
        @app.errorhandler(401)
        def error_401(error):
            return cls.error_401(error)

        @app.errorhandler(403)
        def error_403(error):
            return cls.error_403(error)

        @app.errorhandler(404)
        def error_404(error):
            return cls.error_404(error)

        @app.errorhandler(500)
        def error_500(error):
            return cls.error_500(error)

        @app.errorhandler(503)
        def error_503(error):
            return cls.error_503(error)

    @classmethod
    def error_400(cls, e):
        return cls.render(data={"error": e}), 400

    @classmethod
    def error_401(cls, e):
        return cls.render(data={"error": e}), 401

    @classmethod
    def error_403(cls, e):
        return cls.render(data={"error": e}), 403

    @classmethod
    def error_404(cls, e):
        return cls.render(data={"error": e}), 404

    @classmethod
    def error_500(cls, e):
        return cls.render(data={"error": e}), 500

    @classmethod
    def error_503(cls, e):
        return cls.render(data={"error": e}), 503


"""
Flask-MarcoPolo

$project_name.views.index


"""

from flask import abort
from flask.ext.marcopolo import (MarcoPolo,
                                 route,
                                 flash_error,
                                 flash_success,
                                 flash_info,
                                 set_cookie,
                                 get_cookie)


class Index(MarcoPolo):
    route_base = "/"

    def index(self):
        return self.render()


class Example(MarcoPolo):
    """
    A example
    """

    def index(self):
        flash_success("This is a SUCCESS flash message")
        flash_info("This is an INFO flash message")
        flash_error("This is an ERROR flash message")
        return self.render()

    def text(self):
        return "Just render a normal text"

    def error_401(self):
        abort(401, "This is a 401 Error")

    def error_403(self):
        abort(403, "This is a 403 Error")

    def error_404(self):
        abort(404, "This is a 404 Error")

    def error_500(self):
        abort(500, "This is a 500 Error")




"""
Flask-MarcoPolo

config.py

This module contains config for different environment

Each class based on BaseConf is a different set of configuration for an active environment

For more config options: http://flask.pocoo.org/docs/0.10/config/

- How to use:

It's best to have a mechanism to differentiate dev from prod from staging etc

project_env = "Dev"

my_project = MarcoPolo.init(Flask(__name__), config="project_name.config.%s" % project_env)

"""


class BaseConf(object):
    """
    Base config
    """

    # Flask Default config: http://flask.pocoo.org/docs/0.10/config/
    DEBUG = True
    SECRET_KEY = None

    # Flask-Assets : http://flask-assets.readthedocs.org/
    ASSETS_DEBUG = False
    FLASK_ASSETS_USE_S3 = False
    FLASK_ASSETS_USE_CDN = False

    # Flask-Mail : http://pythonhosted.org/Flask-Mail/
    MAIL_SERVER = 'localhost'
    MAIL_PORT = 25
    MAIL_USE_TLS = False
    MAIL_USE_SSL = False
    MAIL_DEBUG = False
    MAIL_USERNAME = None
    MAIL_PASSWORD = None
    MAIL_DEFAULT_SENDER = None
    MAIL_MAX_EMAILS = None
    MAIL_ASCII_ATTACHMENTS = False


    # Flask-SQLAlchemy : http://pythonhosted.org/Flask-SQLAlchemy/config.html
    SQLALCHEMY_DATABASE_URI = None




class Prod(BaseConf):
    """ Production configuration """

    DEBUG = False


class Dev(BaseConf):
    """ Development configuration """

    DEBUG = True
    SECRET_KEY = "PLEASE_CHANGE_SECRET_KEY"


class Staging(BaseConf):
    """ Staging configuration """
    pass


class Testing(BaseConf):
    """ Configuration for testing """
    pass



"""
Flask-MarcoPolo

Command line tool

flask-marcopolo -c project_name

"""

import os
import argparse
import pkg_resources
import flask_marcopolo

PACKAGE = flask_marcopolo
CWD = os.getcwd()
PROJECTS_TEMPLATES = "projects_templates"


def get_project_dir_path(project_name):
    return "%s/%s" % (CWD, project_name)


def copy_resource(src, dest):
    """
    To copy package data to destination
    """
    dest = (dest + "/" + os.path.basename(src)).rstrip("/")
    if pkg_resources.resource_isdir(PACKAGE.__name__, src):
        if not os.path.isdir(dest):
            os.makedirs(dest)
        for res in pkg_resources.resource_listdir(__name__, src):
            copy_resource(src + "/" + res, dest)
    else:
        if os.path.splitext(src)[1] not in [".pyc"]:
            with open(dest, "wb") as f:
                f.write(pkg_resources.resource_string(__name__, src))


def create_project(project_name, template="default"):
    """
    Create the project
    """
    project_dir = get_project_dir_path(project_name)
    run_file = "%s/run_%s.py" % (CWD, project_name)
    run_tpl = pkg_resources.resource_string(__name__, '%s/run_%s.py.tpl' % (PROJECTS_TEMPLATES, template))

    if not os.path.isdir(project_dir):
        os.makedirs(project_dir)
    else:
        raise OSError("Project dir '%s' exists already " % project_name)

    if not os.path.isfile(run_file):
        with open(run_file, "wb") as f:
            f.write(run_tpl.format(project_name=project_name))

    copy_resource("%s/%s/" % (PROJECTS_TEMPLATES, template), project_dir)


def main():

    print("-" * 80)
    print("%s %s - CLI Tool" % (PACKAGE.__NAME__, PACKAGE.__version__))
    print("")

    parser = argparse.ArgumentParser()
    parser.add_argument("project_name",
                        help="Project name"
                        )
    parser.add_argument("-c", "--create",
                        help="To create a new project."
                             " [flask-marcopolo -c project_name]",
                        action="store_true")
    arg = parser.parse_args()

    try:
        if arg.project_name:
            project_name = arg.project_name
            if arg.create:
                create_project(project_name, "default")
                print("+ PROJECT CREATED SUCCESSFULLY!")
                print("\n+ %s -> %s" % (project_name, get_project_dir_path(project_name)))
                print("\n+ Run 'python run_%s.py' to start the project's server" % project_name)
    except Exception as ex:
        print("- ERROR: %s" % ex.__str__())

    print("")




"""
Flask-MarcoPolo
"""

from setuptools import setup, find_packages
import flask_marcopolo

PACKAGE = flask_marcopolo

setup(
    name=PACKAGE.__NAME__,
    version=PACKAGE.__version__,
    license=PACKAGE.__license__,
    author=PACKAGE.__author__,
    author_email='mardix@github.com',
    description=PACKAGE.__doc__,
    long_description=PACKAGE.__doc__,
    url='http://mardix.github.io/flask-marcopolo/',
    download_url='http://github.com/mardix/flask-marcopolo/tarball/master',
    py_modules=['flask_marcopolo'],
    entry_points=dict(console_scripts=['flask-marcopolo=flask_marcopolo.cmd:main']),
    include_package_data=True,
    packages=find_packages(),
    install_requires=[
        'Flask>=0.10',
        'Flask-Classy>=0.6',
        'Flask-Assets>=0.10',
        'Flask-Mail>=0.9'
    ],

    keywords=['flask', 'templates', 'views', 'classy', 'marcopolo', 'framework'],
    platforms='any',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    zip_safe=False
)
